/***************************************************************************
 *             __________               __   ___.
 *   Open      \______   \ ____   ____ |  | _\_ |__   _______  ___
 *   Source     |       _//  _ \_/ ___\|  |/ /| __ \ /  _ \  \/  /
 *   Jukebox    |    |   (  <_> )  \___|    < | \_\ (  <_> > <  <
 *   Firmware   |____|_  /\____/ \___  >__|_ \|___  /\____/__/\_ \
 *                     \/            \/     \/    \/            \/
 *
 *   Copyright (C) 2007 by Dominik Wenger
 *   $Id: voicefile.h 20623 2009-04-05 12:46:41Z bluebrother $
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
 * KIND, either express or implied.
 *
 ****************************************************************************/


#ifndef VOICEFILE_H
#define VOICEFILE_H

#include <QtCore>
#include "progressloggerinterface.h"

#include "encoders.h"
#include "tts.h"
#include "httpget.h"
#include "rbsettings.h"

#include "wavtrim.h"
#include "voicefont.h"

class VoiceFileCreator :public QObject
{
    Q_OBJECT
public:
    VoiceFileCreator(QObject* parent=0);

    //start creation
    bool createVoiceFile(ProgressloggerInterface* logger);

    // set infos
    void setSettings(RbSettings* sett) { settings = sett;}

    void setMountPoint(QString mountpoint) {m_mountpoint =mountpoint; }
    void setTargetId(int id){m_targetid = id;}
    void setLang(QString name){m_lang =name;}
    void setWavtrimThreshold(int th){m_wavtrimThreshold = th;}

signals:
    void done(bool error);

private slots:
    void abort();
    void downloadDone(bool error);
    void updateDataReadProgress(int read, int total);

private:

    // ptr to encoder, tts and settings
    TTSBase* m_tts;
    EncBase* m_enc;
    RbSettings* settings;
    HttpGet *getter;

    QString filename;  //the temporary file

    QString m_mountpoint;  //mountpoint of the device
    QString m_path;   //path where the wav and mp3 files are stored to
    int m_targetid;  //the target id
    QString m_lang;  // the language which will be spoken
    int m_wavtrimThreshold;

    ProgressloggerInterface* m_logger;

    bool m_abort;
};

#endif

